{
    Respond, the fax number form daemon
    Copyright (C) 2000 Horst F <horstf@gmx.de>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
}
unit DatabaseDescription;

interface

  uses
    Observer;

  type
    {
      Currently supported database types:
        isIniFile: common data object uses TPrivateIniFile
        isWinIni: common data object uses TIniFile for Win.ini access
    }
    TCommonDatabaseType = (isIniFile, isWinIni);

    {
      Describes a database.
      The object supports a link counter. If the object goes to be in use, the
      method "Link" should be called. If a use of the object ends, the method
      "Unlink" should be called. After calling "Unlink" the "IsLink" method
      should be called and if it returns false, the object should be destroyed.
    }
    TDatabaseDescription = class(TObject)
    private
      oName : String;
      oDatabaseType: TCommonDatabaseType;
      oDatabase : TObject;
      oLinks : Integer;  // link counter
    public
      constructor Create(const name: String; const database: TObject;
          const atype: TCommonDatabaseType);
      destructor Destroy; override;
      procedure Link;
      procedure Unlink;
      function IsLinked: Boolean;
      property Name: String read oName;
      property DatabaseType: TCommonDatabaseType read oDatabaseType;
      property Database: TObject read oDatabase;
    end;

implementation

  {
    Creates and fills the object.

    @param name database name
    @param database the database object
    @param atype database type
  }
  constructor TDatabaseDescription.Create(const name: String;
      const database: TObject; const atype: TCommonDatabaseType);
  begin
    inherited Create;
    oName := name;
    oDatabaseType := atype;
    oDatabase := database;
    oLinks := 0;
    ObjectObserver.Observe(self);
  end;

  {
    Destroys the object.
  }
  destructor TDatabaseDescription.Destroy;
  begin
    ObjectObserver.Forget(self);
    inherited Destroy;
  end;

  {
    Increases the link counter.
    Means, the object is used one more time now.
  }
  procedure TDatabaseDescription.Link;
  begin
    Inc(oLinks);
  end;

  {
    Decreases the link counter, if it's greater then zero.
    Means, the object is "freed" on more time.
  }
  procedure TDatabaseDescription.Unlink;
  begin
    if IsLinked then Dec(oLinks);
  end;

  {
    Checks, if the object is in use.

    @return true, if the link counter is greater then zero
  }
  function TDatabaseDescription.IsLinked: Boolean;
  begin
    Result := oLinks > 0;
  end;

end.
